@@ -397,22 +397,31 @@ MimeMessage::contacts(Contact::Type ctype) const noexcept
 	return contacts;
 }
 
-
-
+/*
+ * references() returns the concatenation of the References and In-Reply-To
+ * message-ids (in that order). Duplicates are removed.
+ *
+ * The _first_ one in the list determines the thread-id for the message.
+ */
 std::vector<std::string>
 MimeMessage::references() const noexcept
 {
-	constexpr std::array<const char*, 2> ref_headers = {
-		"References", "In-reply-to",
-	};
-
-	// is ref already in the list?
+	// is ref already in the list? O(n) but with small n.
 	auto is_dup = [](auto&& seq, const std::string& ref) {
 		return seq_some(seq, [&](auto&& str) { return ref == str; });
 	};
 
+	auto is_fake = [](auto&& msgid) {
+		// this is bit ugly; protonmail injects fake References which
+		// can otherwise screw up threading.
+		if (g_str_has_suffix(msgid, "protonmail.internalid"))
+			return true;
+		/* ... */
+		return false;
+	};
+
 	std::vector<std::string> refs;
-	for (auto&& ref_header: ref_headers) {
+	for (auto&& ref_header: { "References", "In-reply-to" }) {
 
 		auto hdr{header(ref_header)};
 		if (!hdr)
@@ -422,12 +431,9 @@ MimeMessage::references() const noexcept
 		refs.reserve(refs.size() + g_mime_references_length(mime_refs));
 
 		for (auto i = 0; i != g_mime_references_length(mime_refs); ++i) {
-
 			const auto msgid{g_mime_references_get_message_id(mime_refs, i)};
-			if (!msgid || is_dup(refs, msgid))
-				continue; // invalid or skip dups
-
-			refs.emplace_back(msgid);
+			if (msgid && !is_dup(refs, msgid) && !is_fake(msgid))
+				refs.emplace_back(msgid);
 		}
 		g_mime_references_free(mime_refs);
 	}
@@ -471,10 +477,22 @@ MimePart::size() const noexcept
 
 	return static_cast<size_t>(g_mime_stream_length(stream));
 }
-
 Option<std::string>
 MimePart::to_string() const noexcept
 {
+	/*
+	 * easy case: text. this automatically handles conversion to utf-8.
+	 */
+	if (GMIME_IS_TEXT_PART(self())) {
+		if (char* txt{g_mime_text_part_get_text(GMIME_TEXT_PART(self()))}; !txt)
+			return Nothing;
+		else
+			return to_string_gchar(std::move(txt)/*consumes*/);
+	}
+
+	/*
+	 * harder case: read from stream manually
+	 */
 	GMimeDataWrapper *wrapper{g_mime_part_get_content(self())};
 	if (!wrapper) { /* this happens with invalid mails */
 		g_debug("failed to create data wrapper");
@@ -487,7 +505,6 @@ MimePart::to_string() const noexcept
 		return Nothing;
 	}
 
-
 	ssize_t buflen{g_mime_data_wrapper_write_to_stream(wrapper, stream)};
 	if (buflen <= 0) { /* empty buffer, not an error */
 		g_object_unref(stream);
@@ -507,9 +524,11 @@ MimePart::to_string() const noexcept
 	buffer.resize(buflen);
 
 	return buffer;
+
 }
 
 
+
 Result<size_t>
 MimePart::to_file(const std::string& path, bool overwrite) const noexcept
 {
