@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2022 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -46,22 +46,7 @@ G_BEGIN_DECLS
  * @return the expanded path as a newly allocated string, or NULL in
  * case of error
  */
-char* mu_util_dir_expand (const char* path)
-	G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
-
-
-/**
- * See g_canonicalize_filename
- *
- * @param filename
- * @param relative_to
- *
- * @return
- */
-char *mu_canonicalize_filename (const gchar *filename,
-                                const gchar *relative_to)
-        G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
-
+char* mu_util_dir_expand(const char* path) G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * guess the maildir; first try $MAILDIR; if it is unset or
@@ -100,42 +85,6 @@ gboolean mu_util_check_dir (const gchar* path, gboolean readable,
 			    gboolean writeable)
 	G_GNUC_WARN_UNUSED_RESULT;
 
-
-/**
- * get our the cache directory, typically, /tmp/mu-<userid>/
- *
- * @return the cache directory; don't free
- */
-const char* mu_util_cache_dir (void) G_GNUC_CONST;
-
-/**
- * create a writeable file and return its file descriptor (which
- * you'll need to close(2) when done with it.)
- *
- * @param path the full path of the file to create
- * @param the mode to open (ie. 0644 or 0600 etc., see chmod(3)
- * @param overwrite should we allow for overwriting existing files?
- *
- * @return a file descriptor, or -1 in case of error. If it's a file
- * system error, 'errno' may contain more info. use 'close()' when done
- * with the file descriptor
- */
-int mu_util_create_writeable_fd (const char* path, mode_t mode,
-				 gboolean overwrite)
-	G_GNUC_WARN_UNUSED_RESULT;
-
-
-/**
- * check if file is local, ie. on the local file system. this means
- * that it's either having a file URI, *or* that it's an existing file
- *
- * @param path a path
- *
- * @return TRUE if the file is local, FALSE otherwise
- */
-gboolean mu_util_is_local_file (const char* path);
-
-
 /**
  * is the current locale utf-8 compatible?
  *
@@ -143,6 +92,19 @@ gboolean mu_util_is_local_file (const char* path);
  */
 gboolean mu_util_locale_is_utf8 (void) G_GNUC_CONST;
 
+/**
+ * get a 'summary' of the string, ie. the first /n/ lines of the
+ * strings, with all newlines removed, replaced by single spaces
+ *
+ * @param str the source string
+ * @param max_lines the maximum number of lines to include in the summary
+ *
+ * @return a newly allocated string with the summary. use g_free to free it.
+ */
+char* mu_str_summarize (const char* str, size_t max_lines)
+    G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
+
+
 /**
  * write a string (assumed to be in utf8-format) to a stream,
  * converted to the current locale
@@ -164,42 +126,21 @@ gboolean mu_util_fputs_encoded (const char *str, FILE *stream);
  */
 gboolean mu_util_print_encoded (const char *frm, ...) G_GNUC_PRINTF(1,2);
 
-/**
- * print a formatted string (assumed to be in utf8-format) to stderr,
- * converted to the current locale
- *
- * @param a standard printf() format string, followed by a parameter list
- *
- * @return TRUE if printing worked, FALSE otherwise
- */
-gboolean mu_util_printerr_encoded (const char *frm, ...) G_GNUC_PRINTF(1,2);
-
 
 /**
- * read a password from stdin (without echoing), and return it.
+ * Try to 'play' (ie., open with it's associated program) a file. On MacOS, the
+ * the program 'open' is used for this; on other platforms 'xdg-open' to do the
+ * actual opening. In addition you can set it to another program by setting the
+ * MU_PLAY_PROGRAM environment variable
  *
- * @param prompt the prompt text before the password
- *
- * @return the password (free with g_free), or NULL
- */
-char* mu_util_read_password (const char *prompt)
-	G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
-
-/**
- * Try to 'play' (ie., open with it's associated program) a file. On
- * MacOS, the the program 'open' is used for this; on other platforms
- * 'xdg-open' to do the actual opening. In addition you can set it to another program
- * by setting the MU_PLAY_PROGRAM environment variable
+ * This requires a 'native' file, see g_file_is_native()
  *
  * @param path full path of the file to open
- * @param allow_local allow local files (ie. with file:// prefix or fs paths)
- * @param allow_remote allow URIs (ie., http, mailto)
  * @param err receives error information, if any
  *
  * @return TRUE if it succeeded, FALSE otherwise
  */
-gboolean mu_util_play (const char *path, gboolean allow_local,
-		       gboolean allow_remote, GError **err);
+gboolean mu_util_play (const char *path, GError **err);
 
 /**
  * Check whether program prog exists in PATH
@@ -334,18 +275,18 @@ enum _MuError {
 	MU_ERROR_XAPIAN_MISSING_DATA         = 17,
 	/* can't get write lock */
 	MU_ERROR_XAPIAN_CANNOT_GET_WRITELOCK = 19,
-        /* could not write */
+	/* could not write */
 	MU_ERROR_XAPIAN_STORE_FAILED	     = 21,
 	/* could not remove */
 	MU_ERROR_XAPIAN_REMOVE_FAILED	     = 22,
 	/* database was modified; reload */
 	MU_ERROR_XAPIAN_MODIFIED             = 23,
-        /* database was modified; reload */
+	/* database was modified; reload */
 	MU_ERROR_XAPIAN_NEEDS_REINDEX        = 24,
-        /* database schema version doesn't match */
-        MU_ERROR_XAPIAN_SCHEMA_MISMATCH      = 25,
-        /* failed to open the database */
-        MU_ERROR_XAPIAN_CANNOT_OPEN          = 26,
+	/* database schema version doesn't match */
+	MU_ERROR_XAPIAN_SCHEMA_MISMATCH      = 25,
+	/* failed to open the database */
+	MU_ERROR_XAPIAN_CANNOT_OPEN          = 26,
 
 	/* GMime related errors */
 
@@ -357,7 +298,7 @@ enum _MuError {
 	MU_ERROR_CONTACTS_CANNOT_RETRIEVE     = 51,
 
 	/* crypto related errors */
-	MU_ERROR_CRYPTO		              = 60,
+	MU_ERROR_CRYPTO			      = 60,
 
 
 	/* File errors */
@@ -395,40 +336,6 @@ typedef enum _MuError MuError;
 gboolean mu_util_g_set_error (GError **err, MuError errcode, const char *frm, ...)
 	G_GNUC_PRINTF(3,4);
 
-/**
- * calculate a 64-bit hash for the given string, based on a combination of the
- * DJB and BKDR hash functions.
- *
- * @param a string
- *
- * @return the hash
- */
-static inline guint64
-mu_util_get_hash (const char* str)
-{
-	guint32 djbhash;
-        guint32 bkdrhash;
-        guint32 bkdrseed;
-        guint64 hash;
-
-        djbhash  = 5381;
-        bkdrhash = 0;
-        bkdrseed = 1313;
-
-        for(unsigned u = 0U; str[u]; ++u) {
-		djbhash  = ((djbhash << 5) + djbhash) + str[u];
-		bkdrhash = bkdrhash * bkdrseed + str[u];
-	}
-
-        hash = djbhash;
-        return (hash<<32) | bkdrhash;
-}
-
-
-
-
-
-
 #define MU_COLOR_RED		"\x1b[31m"
 #define MU_COLOR_GREEN		"\x1b[32m"
 #define MU_COLOR_YELLOW		"\x1b[33m"
