@@ -346,12 +346,9 @@ Mu::vformat(const char* frm, va_list args)
 }
 
 std::string
-Mu::time_to_string(const std::string& frm_, time_t t, bool utc)
+Mu::time_to_string(const char *frm, time_t t, bool utc)
 {
-	/* Temporary hack... https://github.com/djcb/mu/issues/2230 */
-	const auto frm =
-		g_utf8_validate(frm_.c_str(), frm_.length(), {}) ?
-		frm_ : "%c";
+	g_return_val_if_fail(frm, "");
 
 	GDateTime* dt = std::invoke([&] {
 		if (utc)
@@ -366,10 +363,11 @@ Mu::time_to_string(const std::string& frm_, time_t t, bool utc)
 		return {};
 	}
 
-	auto datestr{to_string_opt_gchar(g_date_time_format(dt, frm.c_str()))};
+	frm = frm ? frm : "%c";
+	auto datestr{to_string_opt_gchar(g_date_time_format(dt, frm))};
 	g_date_time_unref(dt);
 	if (!datestr)
-		g_warning("failed to format time with format '%s'", frm.c_str());
+		g_warning("failed to format time with format '%s'", frm);
 
 	return datestr.value_or("");
 }
@@ -585,7 +583,6 @@ Mu::from_lexnum(const std::string& str)
 }
 
 
-
 std::string
 Mu::canonicalize_filename(const std::string& path, const std::string& relative_to)
 {
@@ -602,50 +599,6 @@ Mu::canonicalize_filename(const std::string& path, const std::string& relative_t
 }
 
 
-void
-Mu::allow_warnings()
-{
-	g_test_log_set_fatal_handler(
-	    [](const char*, GLogLevelFlags, const char*, gpointer) { return FALSE; },
-	    {});
-}
-
-
-
-Mu::TempDir::TempDir(bool autodelete): autodelete_{autodelete}
-{
-	GError *err{};
-	gchar *tmpdir = g_dir_make_tmp("mu-tmp-XXXXXX", &err);
-	if (!tmpdir)
-		throw Mu::Error(Error::Code::File, &err,
-				"failed to create temporary directory");
-
-	path_ = tmpdir;
-	g_free(tmpdir);
-
-	g_debug("created '%s'", path_.c_str());
-}
-
-Mu::TempDir::~TempDir()
-{
-	if (::access(path_.c_str(), F_OK) != 0)
-		return; /* nothing to do */
-
-	if (!autodelete_) {
-		g_debug("_not_ deleting %s", path_.c_str());
-		return;
-	}
-
-	/* ugly */
-	GError *err{};
-	const auto cmd{format("/bin/rm -rf '%s'", path_.c_str())};
-	if (!g_spawn_command_line_sync(cmd.c_str(), NULL, NULL, NULL, &err)) {
-		g_warning("error: %s\n", err ? err->message : "?");
-		g_clear_error(&err);
-	} else
-		g_debug("removed '%s'", path_.c_str());
-}
-
 bool
 Mu::locale_workaround() try
 {
