@@ -256,7 +256,7 @@ only marks the first message in the thread.")
 (defvar mu4e-headers-thread-child-prefix '("|>" . "│ ")
   "Prefix for messages in sub threads that do have a following sibling.")
 (defvar mu4e-headers-thread-first-child-prefix '("o " . "⚬ ")
-  "Prefix for messages in sub threads that do not have a following sibling.")
+  "Prefix for the first child messages in a sub thread.")
 (defvar mu4e-headers-thread-last-child-prefix '("L" . "└ ")
   "Prefix for messages in sub threads that do not have a following sibling.")
 (defvar mu4e-headers-thread-connection-prefix '("|" . "│ ")
@@ -352,8 +352,7 @@ This is mostly useful for profiling.")
   "Clear the headers buffer and related data structures.
 Optionally, show TEXT."
   (when (buffer-live-p (mu4e-get-headers-buffer))
-    (setq mu4e~headers-render-start (float-time)
-	  mu4e~view-message nil)
+    (setq mu4e~headers-render-start (float-time))
     (let ((inhibit-read-only t))
       (with-current-buffer (mu4e-get-headers-buffer)
         (mu4e--mark-clear)
@@ -380,14 +379,14 @@ into a string."
          (lambda (cell)
            (if mu4e-use-fancy-chars (cdr cell) (car cell)))))
     (cl-case type
-      ('child         (funcall get-prefix mu4e-headers-thread-child-prefix))
-      ('first-child   (funcall get-prefix mu4e-headers-thread-first-child-prefix))
-      ('last-child    (funcall get-prefix mu4e-headers-thread-last-child-prefix))
-      ('connection    (funcall get-prefix mu4e-headers-thread-connection-prefix))
-      ('blank         (funcall get-prefix mu4e-headers-thread-blank-prefix))
-      ('orphan        (funcall get-prefix mu4e-headers-thread-orphan-prefix))
-      ('single-orphan (funcall get-prefix mu4e-headers-thread-single-orphan-prefix))
-      ('duplicate     (funcall get-prefix mu4e-headers-thread-duplicate-prefix))
+      (child         (funcall get-prefix mu4e-headers-thread-child-prefix))
+      (first-child   (funcall get-prefix mu4e-headers-thread-first-child-prefix))
+      (last-child    (funcall get-prefix mu4e-headers-thread-last-child-prefix))
+      (connection    (funcall get-prefix mu4e-headers-thread-connection-prefix))
+      (blank         (funcall get-prefix mu4e-headers-thread-blank-prefix))
+      (orphan        (funcall get-prefix mu4e-headers-thread-orphan-prefix))
+      (single-orphan (funcall get-prefix mu4e-headers-thread-single-orphan-prefix))
+      (duplicate     (funcall get-prefix mu4e-headers-thread-duplicate-prefix))
       (t              "?"))))
 
 
@@ -568,7 +567,7 @@ and `mu4e-headers-time-format'."
 Otherwise, return the thread-prefix without the subject-text. In
 other words, show the subject of a thread only once, similar to
 e.g. \"mutt\"."
-  (let* ((tinfo  (mu4e-message-field msg :meta))
+  (let* ((tinfo (mu4e-message-field msg :meta))
          (subj (mu4e-msg-field msg :subject)))
     (concat ;; prefix subject with a thread indicator
      (mu4e~headers-thread-prefix tinfo)
@@ -591,7 +590,7 @@ found."
                                field (cdr item)))))
     (funcall func msg)))
 
-(defsubst mu4e~headers-field-value (msg field)
+(defun mu4e~headers-field-value (msg field)
   (let ((val (mu4e-message-field msg field)))
     (cl-case field
       (:subject
@@ -601,7 +600,10 @@ found."
         ;; work-around: emacs' display gets really slow when lines are too long;
         ;; so limit subject length to 600
         (truncate-string-to-width val 600)))
-      (:thread-subject (mu4e~headers-thread-subject msg))
+      (:thread-subject ;; if not searching threads, fall back to :subject
+       (if mu4e-search-threads
+	   (mu4e~headers-thread-subject msg)
+	 (mu4e~headers-field-value msg :subject)))
       ((:maildir :path :message-id) val)
       ((:to :from :cc :bcc) (mu4e~headers-contact-str val))
       ;; if we (ie. `user-mail-address' is the 'From', show
@@ -620,7 +622,7 @@ found."
       (:size (mu4e-display-size val))
       (t (mu4e~headers-custom-field-value msg field)))))
 
-(defun mu4e~headers-truncate-field-fast (val width)
+(defsubst mu4e~headers-truncate-field-fast (val width)
   "Truncate VAL to WIDTH. Fast and somewhat inaccurate."
   (if width
       (truncate-string-to-width val width 0 ?\s truncate-string-ellipsis)
@@ -1556,9 +1558,7 @@ user)."
              (if (eq sortfield mu4e-headers-sort-field)
                  (if (eq mu4e-headers-sort-direction 'ascending)
                      'descending 'ascending)
-               'descending))
-            (mu4e-read-option "Direction: "
-                              '(("ascending" . 'ascending) ("descending" . 'descending))))))
+               'descending)))))
     (setq
      mu4e-headers-sort-field sortfield
      mu4e-headers-sort-direction dir)
@@ -1580,7 +1580,8 @@ last search with the new setting."
 	 (toggles (seq-map
 		   (lambda (cell)
 		     (cons
-		      (concat (car cell) (format" (%s)" (if (cdr cell) "on" "off")))
+		      (concat (car cell) (format" (%s)"
+						(if (symbol-value (cdr cell)) "on" "off")))
 		      (cdr cell))) toggles))
 	 (choice (mu4e-read-option "Toggle setting " toggles)))
     (when choice
