@@ -53,14 +53,31 @@
 (require 'cl-lib)
 
 (require 'mu4e-mark)
-(require 'mu4e-utils)
+(require 'mu4e-helpers)
+(require 'mu4e-contacts)
 (require 'mu4e-headers)
 (require 'mu4e-view)
-(require 'mu4e-vars)
 
-(when mu4e-view-use-old
-  (mu4e-error "iCalender support is not available with the old viewer"))
+
+;;; Configuration
+;;;; Calendar
 
+(defgroup mu4e-icalendar nil
+  "Icalendar related settings."
+  :group 'mu4e)
+
+(defcustom mu4e-icalendar-trash-after-reply nil
+  "If non-nil, trash the icalendar invitation after replying."
+  :type 'boolean
+  :group 'mu4e-icalendar)
+
+(defcustom mu4e-icalendar-diary-file nil
+  "If non-nil, the file in which to add events upon reply."
+  :type '(choice (const :tag "Do not insert a diary entry" nil)
+                 (string :tag "Insert a diary entry in this file"))
+  :group 'mu4e-icalendar)
+
+
 ;;;###autoload
 (defun mu4e-icalendar-setup ()
   "Perform the necessary initialization to use mu4e-icalendar."
@@ -79,7 +96,7 @@
 (defun mu4e~icalendar-has-email (email list)
   "Check that EMAIL is in LIST."
   (let ((email (downcase email)))
-    (cl-find-if (lambda (c) (let ((e (cdr c)))
+    (cl-find-if (lambda (c) (let ((e (mu4e-contact-email c)))
                               (and (stringp e) (string= email (downcase e)))))
                 list)))
 
@@ -119,18 +136,19 @@
                   (mu4e-sent-messages-behavior 'delete)
                   (mu4e-compose-reply-recipients 'sender)
                   (ical-msg (cl-copy-list msg)))
-              ;; Make sure the reply is sent to the organiser.
+              ;; Make sure the reply is sent to email of the organiser with proper name.
               (let* ((organizer (gnus-icalendar-event:organizer event))
-                     (reply-to (plist-get msg :reply-to))
-                     (name (or (caar reply-to)
-                               (caar (plist-get msg :from))))
-                     (email (cons name organizer)))
+                     (reply-to (car (plist-get msg :reply-to)))
+                     (from     (car (plist-get msg :from)))
+                     (name (or  (plist-get reply-to :name)
+                                (plist-get from :name))))
+                ;; Add :reply-to field when incomplete or absent
                 (unless (or (string= organizer "")
                             (mu4e~icalendar-has-email organizer reply-to))
-                  (plist-put ical-msg :reply-to (cons email reply-to))))
-              (plist-put ical-msg :subject
-                         (concat (capitalize (symbol-name status))
-                                 ": " (gnus-icalendar-event:summary event)))
+                  (plist-put ical-msg :reply-to `((:name ,name :email ,organizer))))
+                (plist-put ical-msg :subject
+                           (concat (capitalize (symbol-name status))
+                                   ": " (gnus-icalendar-event:summary event))))
               (mu4e~compose-handler
                'reply ical-msg
                `((:buffer-name ,ical-name
