@@ -697,9 +697,10 @@ determine which browser function to use."
   "Show current MSG in an embedded xwidget, if available."
   (unless (fboundp 'xwidget-webkit-browse-url)
     (mu4e-error "No xwidget support available"))
-  (let ((browse-url-browser-function
-	 (lambda (url &optional _rest)
-	   (xwidget-webkit-browse-url url))))
+  (let ((browse-url-handlers nil)
+        (browse-url-browser-function
+         (lambda (url &optional _rest)
+           (xwidget-webkit-browse-url url))))
     (mu4e-action-view-in-browser msg)))
 
 (defun mu4e~view-render-buffer (msg)
@@ -1300,12 +1301,13 @@ the third MIME-part."
   (interactive)
   ;; This function assumes `gnus-article-mime-handle-alist' is sorted by
   ;; pertinence, i.e. the first HTML part found in it is the most important one.
-  (if-let ((html-part
-	    (seq-find (lambda (handle)
-			(equal (mm-handle-media-type (cdr handle)) "text/html"))
-		      gnus-article-mime-handle-alist)))
-      (gnus-article-inline-part (car html-part))
-    (mu4e-warn "No html part in this message")))
+  (save-excursion
+    (if-let ((html-part
+	      (seq-find (lambda (handle)
+			  (equal (mm-handle-media-type (cdr handle)) "text/html"))
+		        gnus-article-mime-handle-alist)))
+        (gnus-article-inline-part (car html-part))
+      (mu4e-warn "No html part in this message"))))
 
 (defun mu4e-process-file-through-pipe (path pipecmd)
   "Process file at PATH through a pipe with PIPECMD."
